# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import os
import tempfile
import path
import json
import uuid

# ############################################################################################################# #
#                                                                                                               #
#    Text/Plain Output Map Format                                                                               #
#                                                                                                               #
# ############################################################################################################# #
class OutputFileParameter(object):

    def __init__(self, par_name, d, template_vars_for_param_types=None, wps_execution_shared_dir=None, uploader=None):
        #{"type": "string", "description": "xml OAA output file", "filepath" : "%workdir\\\\output_file.xml" }
        self._name=par_name
        self._type=None
        self._description = None
        self._title = None
        self._filepath = None
        self._output_mime_type = None

        self._wps_execution_shared_dir = wps_execution_shared_dir
        self._uploader = uploader
        self._backup_on_wps_execution_shared_dir = None
        self._upload_data = None
        self._upload_data_root = None
        self._publish_as_layer = None
        self._publish_layer_name = None
        self._publish_default_style = None
        self._publish_target_workspace = None
        self._publish_metadata = None

        for k,v in d.items():
            if hasattr(self, "_" + k):
                if template_vars_for_param_types != None and isinstance(v, basestring):
                    for var, val in template_vars_for_param_types.items():
                        if var in v:
                            v=v.replace("%" + var,val)

                setattr(self, "_" + k, v)

        self._filepath = path.path(self._filepath)

    def get_name(self):
        return self._name

    def as_json_string(self):
        #{"type": "string", "description": "A persons surname", "max": 1, "default": "Meier"}
        res={}
        attrib_to_convert = ['_type', '_description', '_title', '_output_mime_type', '_publish_as_layer', '_publish_layer_name', '_publish_default_style', '_publish_target_workspace']
        attribute_list = [a for a in dir(self) if not a.startswith('__') and not callable(getattr(self, a))]
        attribute_list_filtered = [x for x in attribute_list if x in attrib_to_convert]
        for a in attribute_list_filtered:
            res[a[1:]] = getattr(self, a)
        return json.dumps(res)

    def get_value(self):
        if self._backup_on_wps_execution_shared_dir != None and self._backup_on_wps_execution_shared_dir.lower() == "true" and self._wps_execution_shared_dir != None:
            unique_dirname = str(uuid.uuid4())
            bkp_dir = path.path(self._wps_execution_shared_dir + "/" + unique_dirname)
            bkp_dir.makedirs()
            dst = bkp_dir.abspath() + "/" + self._filepath.basename()

            self._filepath.copy(dst)
            dst = path.path(dst)

            return dst.text()
        elif self._upload_data != None and self._upload_data.lower() == "true" and self._uploader != None:
            unique_dirname = str(uuid.uuid4())
            bkp_dir = path.path(tempfile.gettempdir() + '/' + unique_dirname)
            bkp_dir.makedirs()
            dst = bkp_dir.abspath() + '/' + self._filepath.basename()

            self._filepath.copy(dst)
            dst = path.path(dst)

            src_path = os.path.abspath(os.path.join(dst.abspath(), os.pardir))
            if self._upload_data_root:
                unique_dirname = self._upload_data_root + '/' + unique_dirname
            self._uploader.Upload(hostdir=unique_dirname, text='', binary='*.*', src=src_path)

            return self._filepath.text()
        else:
            return self._filepath.text()

    def get_type(self):
        return "textual"

    def get_description(self):
        return self._description

    def get_title(self):
        return self._title

    def get_output_mime_type(self):
        return self._output_mime_type

    def is_publish_as_layer(self):
        return (self._publish_as_layer != None and self._publish_as_layer.lower() == "true")

    def get_publish_layer_name(self):
        return self._publish_layer_name

    def get_publish_default_style(self):
        return self._publish_default_style

    def get_publish_target_workspace(self):
        return self._publish_target_workspace

    def get_metadata(self):
        if self._publish_metadata != None:
            metadata_file = path.path(self._publish_metadata)

            if metadata_file.isfile():
                return metadata_file.text()
        return ' '

# ############################################################################################################# #
#                                                                                                               #
#    RAW File Output Map Format                                                                                 #
#                                                                                                               #
# ############################################################################################################# #
class RawFileParameter(object):

    def __init__(self, par_name, d, template_vars_for_param_types=None, wps_execution_shared_dir=None, uploader=None):
        #{"type": "string", "description": "xml OAA output file", "filepath" : "%workdir\\\\output_file.xml" }
        self._name=par_name
        self._type=None
        self._description = None
        self._title = None
        self._filepath = None
        self._output_mime_type = None

        self._wps_execution_shared_dir = wps_execution_shared_dir
        self._uploader = uploader
        self._backup_on_wps_execution_shared_dir = None
        self._upload_data = None
        self._upload_data_root = None
        self._publish_as_layer = None
        self._publish_layer_name = None
        self._publish_default_style = None
        self._publish_target_workspace = None
        self._publish_metadata = None

        for k,v in d.items():
            if hasattr(self, "_" + k):
                if template_vars_for_param_types != None and isinstance(v, basestring):
                    for var, val in template_vars_for_param_types.items():
                        if var in v:
                            v=v.replace("%" + var,val)

                setattr(self, "_" + k, v)

        self._filepath = path.path(self._filepath)

    def get_name(self):
        return self._name

    def as_json_string(self):
        #{"type": "string", "description": "A persons surname", "max": 1, "default": "Meier"}
        res={}
        attrib_to_convert = ['_type', '_description', '_title', '_output_mime_type', '_publish_as_layer', '_publish_layer_name', '_publish_default_style', '_publish_target_workspace']
        attribute_list = [a for a in dir(self) if not a.startswith('__') and not callable(getattr(self, a))]
        attribute_list_filtered = [x for x in attribute_list if x in attrib_to_convert]
        for a in attribute_list_filtered:
            res[a[1:]] = getattr(self, a)
        return  json.dumps(res)

    def get_value(self):
        if self._backup_on_wps_execution_shared_dir != None and self._backup_on_wps_execution_shared_dir.lower() == "true" and self._wps_execution_shared_dir != None:
            unique_dirname = str(uuid.uuid4())
            bkp_dir = path.path(self._wps_execution_shared_dir + "/" + unique_dirname)
            bkp_dir.makedirs()
            dst = bkp_dir.abspath() + "/" + self._filepath.basename()

            self._filepath.copy(dst)
            dst = path.path(dst)

            return dst
        elif self._upload_data != None and self._upload_data.lower() == "true" and self._uploader != None:
            unique_dirname = str(uuid.uuid4())

            if self._upload_data_root:
                unique_dirname = self._upload_data_root + '/' + unique_dirname
            src_path = os.path.abspath(os.path.join(self._filepath.abspath(), os.pardir))
            basename = os.path.basename(self._filepath.abspath())
            basename = os.path.splitext(basename)[0]
            self._uploader.Upload(hostdir=unique_dirname, text='', binary=basename+'*.*', src=src_path)

            return path.path(unique_dirname + "/" + self._filepath.basename())
        else:
            return self._filepath

    def get_type(self):
        return self._type

    def get_description(self):
        return self._description

    def get_title(self):
        return self._title

    def get_output_mime_type(self):
        return self._output_mime_type

    def is_publish_as_layer(self):
        return (self._publish_as_layer != None and self._publish_as_layer.lower() == "true")

    def get_publish_layer_name(self):
        return self._publish_layer_name

    def get_publish_default_style(self):
        return self._publish_default_style

    def get_publish_target_workspace(self):
        return self._publish_target_workspace

    def get_metadata(self):
        if self._publish_metadata != None:
            metadata_file = path.path(self._publish_metadata)

            if metadata_file.isfile():
                return metadata_file.text()
        return ' '


# ############################################################################################################# #
#                                                                                                               #
#    OWC Json Output Map Format                                                                                 #
#                                                                                                               #
# ############################################################################################################# #
class OWCFileParameter(object):

    def __init__(self, par_name, d, parameters_types_defs, template_vars_for_param_types=None, wps_execution_shared_dir=None, uploader=None):
        #{"type": "string", "description": "xml OAA output file", "filepath" : "%workdir\\\\output_file.xml" }
        self._name=par_name
        self._type=None
        self._description = None
        self._title = None
        self._output_mime_type = None

        self._layers_to_publish = None

        self._wps_execution_shared_dir = wps_execution_shared_dir
        self._uploader = uploader
        self._backup_on_wps_execution_shared_dir = None
        self._upload_data = None
        self._upload_data_root = None
        self._publish_as_layer = "true"
        self._publish_layer_name = None
        self._publish_metadata = None

        for k,v in d.items():
            if hasattr(self, "_" + k):
                if template_vars_for_param_types != None and isinstance(v, basestring):
                    for var, val in template_vars_for_param_types.items():
                        if var in v:
                            v=v.replace("%" + var,val)

                setattr(self, "_" + k, v)

        self._files_to_publish  = ''
        self._default_styles    = ''
        self._target_workspaces = ''

        if self._layers_to_publish != None:
            self._parameters_types_defs = parameters_types_defs

            layer_names = self._layers_to_publish.split(';')
            for name in layer_names:
                if self._parameters_types_defs[name] != None:
                    publish_layer_name = self._parameters_types_defs[name].get('publish_layer_name')
                    publish_layer_name = publish_layer_name if publish_layer_name != None else ' '

                    publish_default_style = self._parameters_types_defs[name].get('publish_default_style')
                    publish_default_style = publish_default_style if publish_default_style != None else ' '

                    publish_target_workspace = self._parameters_types_defs[name].get('publish_target_workspace')
                    publish_target_workspace = publish_target_workspace if publish_target_workspace != None else ' '

                    self._files_to_publish += publish_layer_name + ";"
                    self._default_styles   += publish_default_style + ";"
                    self._target_workspaces+= publish_target_workspace + ";"

    def get_name(self):
        return self._name

    def as_json_string(self):
        #{"type": "string", "description": "A persons surname", "max": 1, "default": "Meier"}
        res={}
        attrib_to_convert = ['_type', '_description', '_title', '_output_mime_type', '_publish_as_layer', '_publish_layer_name', '_files_to_publish', '_default_styles', '_target_workspaces']
        attribute_list = [a for a in dir(self) if not a.startswith('__') and not callable(getattr(self, a))]
        attribute_list_filtered = [x for x in attribute_list if x in attrib_to_convert]
        for a in attribute_list_filtered:
            res[a[1:]] = getattr(self, a)
        return  json.dumps(res)

    def get_value(self):
        if self._backup_on_wps_execution_shared_dir != None and self._backup_on_wps_execution_shared_dir.lower() == "true" and self._wps_execution_shared_dir != None:
            unique_dirname = str(uuid.uuid4())
            bkp_dir = path.path(self._wps_execution_shared_dir + "/" + unique_dirname)
            bkp_dir.makedirs()

            tokens = self._files_to_publish.split(';')

            files_to_publish = ""
            for token in tokens:
                filepath = path.path(token)
                dst = bkp_dir.abspath() + "/" + filepath.basename()
                filepath.copy(dst)
                dst = path.path(dst)

                if len(files_to_publish) > 0:
                    files_to_publish = files_to_publish + ";"
                files_to_publish = files_to_publish + dst.abspath()

            return files_to_publish
        elif self._upload_data != None and self._upload_data.lower() == "true" and self._uploader != None:
            unique_dirname = str(uuid.uuid4())
            bkp_dir = path.path(tempfile.gettempdir() + '/' + unique_dirname)
            bkp_dir.makedirs()

            tokens = self._files_to_publish.split(';')

            if self._upload_data_root:
                unique_dirname = self._upload_data_root + '/' + unique_dirname

            files_to_publish = ""
            for token in tokens:
                filepath = path.path(token)
                dst = bkp_dir.abspath() + '/' + filepath.basename()
                filepath.copy(dst)
                dst = path.path(dst)

                if len(files_to_publish) > 0:
                    files_to_publish = files_to_publish + ';'
                files_to_publish = files_to_publish + '/' + unique_dirname + '/' + filepath.basename()

            self._uploader.Upload(hostdir=unique_dirname, text='', binary='*.*', src=bkp_dir.abspath())

            return files_to_publish
        else:
            return self._files_to_publish

    def get_type(self):
        return self._type

    def get_description(self):
        return self._description

    def get_title(self):
        return self._title

    def get_output_mime_type(self):
        return self._output_mime_type

    def is_publish_as_layer(self):
        return (self._publish_as_layer != None and self._publish_as_layer.lower() == "true")

    def get_publish_layer_name(self):
        return self._publish_layer_name

    def get_publish_default_style(self):
        return self._default_styles

    def get_publish_target_workspace(self):
        return self._target_workspaces

    def get_metadata(self):
        if self._publish_metadata != None:
            metadata_file = path.path(self._publish_metadata)

            if metadata_file.isfile():
                return metadata_file.text()
        return ' '



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import computation_job_input

class ComputationJobConst(computation_job_input.ComputationJobInput):

    def __init__(self, name, input_type, title, descr, value):
        super(ComputationJobConst, self).__init__(name, input_type, title, descr)
        self.set_value( value)




# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import input_parameter

class CommandLineParameter(input_parameter.InputParameter):
    def __init__(self, name):
        input_parameter.InputParameter.__init__(self, name)
        self._template=None

    def get_cmd_line(self):
        cmd=''
        hasName = False
        hasValue = False
        if 'name' in self._template:
            self._template = self._template.replace("name", "%s")
            hasName = True
        if 'value' in self._template:
            self._template = self._template.replace("value","%s")  
            hasValue = True
        for v in self._value:
            if (hasName and hasValue):
                cmd += self._template % (self.get_name(), v) + " "
            elif hasName:
                cmd += self._template % (self.get_name()) + " "
            elif hasValue:
                cmd += self._template % (v) + " "
            else:
                raise Exception("Bad template for command line parameter " + self.get_name())
        return cmd 

class CommandLineParameterConst(CommandLineParameter):
    def __init__(self, name):
        CommandLineParameter.__init__(self, name)
        self._template=None

    def inject_values(self, paremeters_types_defs):
        super(CommandLineParameterConst, self).inject_values(paremeters_types_defs)
        if type(self._value) is not list:
            self._value = [self._value]

    def set_actual_value(self, value):
        """override this function because in this case the param value is set in the constructor not with this function"""
        pass



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

class Upload(object):
    def __init__(self, host, username, password, id):
        self.callbacks={}
        self.id = id
        self.host = host
        self.username = username
        self.password = password

    def Upload(self, hostdir='.', text='*.*', binary='', src='.'):
        """
        Upload a set of files.
        Source files are found in the directory named by `src`
        (and its subdirectories recursively).  The files are uploaded
        to the directory named by `hostdir` on the remote host.
        Files that match one of the space-separated patterns in `text`
        are uploaded as text files, those that match the patterns in
        `binary` are uploaded as binary files.
        """
        pass



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import json
import string

import path

import input_parameters
import input_parameter


class ConfigFileParameter(input_parameter.InputParameter):
    def __init__(self, name): 
        input_parameter.InputParameter.__init__(self, name)
        self._filepath=None

    def inject_values(self, paremeters_types_defs):
        super(ConfigFileParameter, self).inject_values(paremeters_types_defs)
        self._filepath = path.path(self._filepath)

    def get_cmd_line(self):
        self.update_file()
        return ' '

    #todo: rename to write file
    def update_file(self):
        pass



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import json
import path

import computational_job_input_action

class ComputationalJobInputActionUpdateJSONFile(computational_job_input_action.ComputationalJobInputAction):


    def __init__(self, input_ref, target_json_file, json_path_expr, source_template_json_file):
        super(ComputationalJobInputActionUpdateJSONFile, self).__init__()
        #if not type(input_refs) is list:
        #    self._input_refs = [input_refs]
        #else:
        self._input_ref = input_ref
        #file path that will be updated
        #self._target_json_file = target_json_file
        self._target_json_file = target_json_file if isinstance(target_json_file, path.path) else path.path(target_json_file)
        # Python-like expression to reference the attribute in the json file to be set
        self.jsonpath_expr = json_path_expr
        # If target file doesn't exists it will be created using the template at this file path
        self._config_file_template = source_template_json_file if isinstance(source_template_json_file, path.path) else path.path(source_template_json_file)
        

    def set_inputs(self, inputs):
        if self._input_ref in inputs.names():
            if not self.exists() and self._config_file_template!= None:
                self._config_file_template.copyfile( self._target_json_file )
                self.update_file(inputs)
            elif not self.exists() and self._config_file_template== None:
                raise Exception("Cannot find target JSON file " + str(self._target_json_file) )
            else:
                self.update_file(inputs)

    def update_file(self, inputs):
        json_text = self._target_json_file.text()
        j=json.loads(json_text)
        exec( "j" + self.jsonpath_expr + "=" + str(  inputs[self._input_ref].get_value_as_JSON_literal() ) )
        json.dump( j, self._target_json_file.open('w'), indent=4 )

    def exists(self):
        return self._target_json_file.exists()




# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import computation_job_input

class ComputationJobParam(computation_job_input.ComputationJobInput):

    def __init__(self, name, input_type, title, descr, default=None, formatter=None, min_occurencies=0, max_occurencies=1, input_mime_type=None):
        super(ComputationJobParam, self).__init__(name, input_type, title, descr, default, formatter, input_mime_type)
        self._min = min_occurencies
        self._max = max_occurencies

    def validate(self):
        if not (self._min <= len(self._value) and len(self._value) <= self._max):
            raise TypeError("Actual value for parameter has wrong multiplicity")
        return super(ComputationJobParam, self).validate()








# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

"""
SFtpUpload

Based on paramiko sftp client.
"""

import paramiko, pickle, sys, md5, os, string
import logging      # if not std, http://www.red-dove.com/python_logging.html
import path         # http://www.jorendorff.com/articles/python/path
import upload


class EzFtp:
    """
    A simplified interface to ftplib.

    Lets you use full pathnames, with server-side
    directory management handled automatically.
    """
    def __init__(self, ftp):
        self.ftp = ftp
        self.serverDir = ''

    def setRoot(self, dir):
        """
        Set the remote directory that we'll call the root.
        """
        self.cd(dir, create=True)
        self.ftp.exec_command('cd ..; pwd')
        self.ftp.exec_command('cd '+dir+'; pwd')

    def cd(self, dir, create=1):
        """
        Change the directory on the server, if need be.
        If create is true, directories are created if necessary to get to the full path.
        Returns true if the directory is changed.
        """
        if dir != self.serverDir:
            # Move up to the common root.
            while not dir.startswith(self.serverDir):
                logging.info("ftpcd ..")
                self.ftp.exec_command('cd ..; pwd')
                self.serverDir = os.path.split(self.serverDir)[0]
            # Move down to the right directory
            doDirs = dir[len(self.serverDir):]
            for d in string.split(doDirs, os.sep):
                if d:
                    try:
                        logging.info("ftpcd %s" % d)
                        self.ftp.exec_command('cd '+d+'; pwd')
                    except:
                        if create:
                            logging.info("ftpmkdir %s" % d)
                            try:
                                self.ftp.mkdir(d)
                            except:
                                pass
                            self.ftp.exec_command('cd '+d+'; pwd')
                        else:
                            return 0
                    self.serverDir = os.path.join(self.serverDir, d)
        return 1

    def putasc(self, this, that):
        """
        Put a text file to the server.
        """
        thatDir, thatFile = os.path.split(that)
        self.cd(thatDir)
        f = open(this, "r")
        logging.info("ftpstorasc %s" % that)
        self.ftp.putfo(f,thatFile)

    def putbin(self, this, that):
        """
        Put a binary file to the server.
        """
        thatDir, thatFile = os.path.split(that)
        self.cd(thatDir)
        f = open(this, "rb")
        logging.info("ftpstorbin %s" % that)
        self.ftp.putfo(f,thatFile)

    def delete(self, that):
        """
        Delete a file on the server.
        """
        thatDir, thatFile = os.path.split(that)
        if self.cd(thatDir, 0):
            logging.info("ftpdel %s" % that)
            try:
                self.ftp.remove(thatFile)
            except:
                pass

    def quit(self):
        """
        Quit.
        """
        self.ftp.close()


class SFtpUpload(upload.Upload):
    """
    Provides intelligent FTP uploading of files, using MD5 hashes to track
    which files have to be uploaded.  Each upload is recorded in a local
    file so that the next upload can skip the file if its contents haven't
    changed.  File timestamps are ignored, allowing regenerated files to
    be properly uploaded only if their contents have changed.

    Call `setHost` and `setMd5File` to establish the settings for a session,
    then `upload` for each set of files to upload.  If you want to have
    removed local files automatically delete the remote files, call
    `deleteOldFiles` once, then `finish` to perform the closing bookkeeping.

    ::

        fu = SFtpUpload(config, 'ftp.myhost.com', 'myusername', 'password')
        fu.setHost('ftp.myhost.com', 'myusername', 'password')
        fu.setMd5File('myhost.md5') # optional
        fu.upload(
            hostdir='www', src='.',
            text='*.html *.css', binary='*.gif *.jpg'
        )
        # more upload() calls can go here..
        fu.deleteOldFiles()
        fu.finish()

    """

    def __init__(self, host, username, password, id="master"):
        upload.Upload.__init__(self, host, username, password, id)

        self.setHost(host, username, password)
        self.ftp = None
        self.ezftp = None
        self.md5file = None
        self.md5DictIn = {}
        self.md5DictOut = {}
        self.md5DictUp = {}

    def setHost(self, host, username, password):
        """
        Set the host, the username and password.
        """
        try:
            hoststr, portstr = host.split(':')
        except:
            hoststr = host
            portstr = None
        if portstr:
            transport = paramiko.Transport((hoststr, int(portstr)))
        else:
            transport = paramiko.Transport((hoststr, 22))
        transport.connect(username = username, password = password)
        self.ftp = paramiko.SFTPClient.from_transport(transport)

    def setMd5File(self, md5file):
        """
        Assign a filename to use for the MD5 tracking.
        """
        self.md5file = md5file
        if self.md5file:
            try:
                inf = open(self.md5file, "r")
                self.md5DictIn = pickle.load(inf)
                self.md5DictUp.update(self.md5DictIn)
                inf.close()
            except IOError:
                self.md5DictIn = {}

    def Upload(self,
               hostdir='.',
               text='*.*',
               binary='',
               src='.'
               ):
        """
        Upload a set of files.
        Source files are found in the directory named by `src`
        (and its subdirectories recursively).  The files are uploaded
        to the directory named by `hostdir` on the remote host.
        Files that match one of the space-separated patterns in `text`
        are uploaded as text files, those that match the patterns in
        `binary` are uploaded as binary files.

        This method can be called a number of times to upload different
        sets of files to or from different directories within the same
        FtpUpload session.
        """

        if not self.ezftp:
            if not self.ftp:
                self.setHost(self.host, self.username, self.password)
            self.ezftp = EzFtp(self.ftp)

        if hostdir != '.':
            self.ezftp.setRoot('/')
            self.ezftp.setRoot(hostdir)

        # patdict is a dict of fnmatch patterns to ftp function.
        patdict = {}
        for pat in text.split():
            patdict[pat] = self.ezftp.putasc
        for pat in binary.split():
            patdict[pat] = self.ezftp.putbin

        # Walk the tree, putting files to the ezftp.
        srcpath = path.path(src)
        for thispath in srcpath.walkfiles():
            thatpath = hostdir + os.sep + srcpath.relpathto(thispath)
            logging.info("thatpath %s" % thatpath)
            thatpathstr = str(thatpath)
            # Compute this file's MD5 fingerprint
            m = md5.new()
            f = open(thispath, "rb")
            for l in f.readlines():
                m.update(l)
            thisMd5 = m.hexdigest()

            # What was the last MD5 fingerprint?
            thatMd5 = self.md5DictIn.get(thatpathstr, '')

            # If the current file is different, then put it to the server.
            if thisMd5 != thatMd5:
                # Find the pattern the file matches, and use the ftp function
                # from the map.
                for pat in patdict.keys():
                    if thispath.fnmatch(pat):
                        ftpfn = patdict[pat]
                        ftpfn(thispath, thatpath)

            # Remember the new fingerprint.
            self.md5DictOut[thatpathstr] = thisMd5
            self.md5DictUp[thatpathstr] = thisMd5

    def deleteOldFiles(self):
        """
        Delete any remote files that we have uploaded previously but
        that weren't considered in this FtpUpload session.  This doesn't
        touch files that exist on the remote host but were never uploaded
        by this module.
        """

        # Files in md5DictIn but not in md5DictOut must have been removed.
        for this in self.md5DictIn:
            if this not in self.md5DictOut:
                self.ezftp.delete(this)
                del self.md5DictUp[this]

    def finish(self):
        """
        Do our final bookkeeping.
        """
        # Done with ftp'ing.
        self.ezftp.quit()

        # Write the md5 control file out for next time.
        if self.md5file:
            outf = open(self.md5file, "w")
            pickle.dump(self.md5DictUp, outf)
            outf.close()




from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA
import base64
import sys

def decrypt(message, externKey, passphrase):
    privatekey = open(externKey, "r")
    rsa_key = RSA.importKey(privatekey, passphrase=passphrase)
    # verifier = PKCS1_v1_5.new(rsa_key)
    # h = SHA.new(message)
    decriptedData = rsa_key.decrypt(base64.b64decode(message))
    print(decriptedData)

if __name__ == "__main__":
    if len(sys.argv) == 4:
        decrypt(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        print("Usage: python decrypt.py password path/to/rsakey.pem passphrase")



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import socket
import logging
import traceback
import urllib
import pickle
import sleekxmpp
import busIndipendentMessages
import xmppMessages
import bus
import sys
import time


class XMPPBus(bus.Bus):
    def __init__(self, config, service_name, service_name_namespace, id="master"):
        bus.Bus.__init__(self, id)

        self.config=config
        self.address=(config.get("DEFAULT", "address"), config.get("DEFAULT", "port")) # As address use mnemonic names
        self.domain = config.get("DEFAULT", "domain")
        self.MUC_name = config.get("DEFAULT", "mucService")
        self.username=config.get("DEFAULT", "user")
        self.password=config.get("DEFAULT", "password")
        self.nameSpacePassword = config.get("DEFAULT", "mucServicePassword")

        self._service_name = service_name
        self._service_name_namespace = service_name_namespace
        self._fully_qualified_service_name = self._service_name_namespace + "." + self._service_name

        self.xmpp = sleekxmpp.ClientXMPP(self._get_JId(), self.password)
        self.xmpp.register_plugin('xep_0004') # Data forms
        self.xmpp.register_plugin('xep_0030') # Service Discovery
        self.xmpp.register_plugin('xep_0045') # MUC
        self.xmpp.register_plugin('xep_0060') # MUC
        self.xmpp.register_plugin('xep_0077') # In-band Registration
        self.xmpp.register_plugin('xep_0078') # Non-SASL Authentication
        self.xmpp.register_plugin('xep_0199') # XMPP Ping
        self.xmpp.register_plugin('xep_0249') # Direct MUC invitation
        self.xmpp.add_event_handler('register', self._register)
        self.xmpp.add_event_handler('session_start', self._startService)
        self.xmpp.add_event_handler('message', self._handleXMPPSignal)


    def clone_for_process(self, id):
        c=XMPPBus(self.config, self._service_name, self._service_name_namespace, id)
        return c


    def _get_JId(self):
        # default.Service@whale.nurc.nato.int/master@WW717962
        # return ''.join([self._fully_qualified_service_name, '@', self.domain, '/', self.id, '@', socket.gethostname()])
        return ''.join([self.username, '@', self.domain, '/', self.id, '@', socket.gethostname()])

    def _get_MUC_JId(self):
        return self._service_name_namespace + "@" + self.MUC_name

    def _MUC_service_nickname(self):
        return '%s@%s' % (self._service_name, str(self.xmpp.boundjid).split('@', 1).pop())


    def CheckServerIdentity(self, serverId):
        pass

    def get_fully_qualified_service_name(self):
        return self._fully_qualified_service_name

    def Listen(self):
        logging.info('Trying to connect...')
        for x in xrange(5):
            if self.xmpp.connect(self.address, use_tls=False, use_ssl=True):
                logging.info('State: %s' % (str(self.state())))
                if self.state() == 'connected':
                    try:
                        self.xmpp.send_presence()
                        # self.xmpp.get_roster()
                        self.xmpp.process(block = True)
                        break
                    except:
                        stack_trace = traceback.format_exc(sys.exc_info())
                        logging.exception( "Service "+str(self._service_name)+" Exception: "+str(stack_trace))
                    finally:
                        self.xmpp.disconnect()
                else:
                    logging.info('Cannot connect to xmpp server. Tentative # ' + str(x+1))
                    time.sleep(2)

    def _startService(self, event):
        # once the connection is established, immidiatelly join the MUC, no notification is sent to the calling class (serviceBot and processBot)
        self.JoinMUC()

    def JoinMUC(self):
        self.xmpp.plugin['xep_0045'].joinMUC(self._get_MUC_JId(), self._MUC_service_nickname(), password=self.nameSpacePassword)

    def _handleXMPPSignal(self, msg):
        imsg=self.CreateIndipendentMessage(msg)
        if (imsg is not None):
            self.callbacks[type(imsg)](imsg)

    def CreateIndipendentMessage(self, msg):
        if msg['type'] in ('normal', 'chat'):
            payload = msg['body']
            origin = msg['from']

            logging.info('Received XMPP bus signal from %s: "%s"' % (origin, payload))

            #todo: check for role=moderator
            role = 'visitor'
            roster = self.xmpp.plugin['xep_0045'].getRoster(self._get_MUC_JId())
            if origin.resource in roster:
                role = self.xmpp.plugin['xep_0045'].rooms[self._get_MUC_JId()][origin.resource]['role']
            elif origin.user in roster:
                role = self.xmpp.plugin['xep_0045'].rooms[self._get_MUC_JId()][origin.user]['role']

            logging.info("handle message from WPS Role " + str(role))
            if role != 'visitor':
                if ("topic=request" in payload):
                    logging.info("handle request message from WPS " + str(payload))
                    variables = dict([tuple(each.strip().split('=')) for each in payload.split('&')])
                    requestParams=pickle.loads(urllib.unquote(variables['message'])) if variables['message'] is not None else None
                    #print str(variables)
                    return busIndipendentMessages.ExecuteMessage( msg['from'], variables['id'], variables['baseURL'], requestParams )
                elif ("topic=invite" in payload):
                    logging.info("handle invite message from WPS " + str(payload))
                    return busIndipendentMessages.InviteMessage(payload, msg['from'])
                elif ("topic=finish" in payload):
                    logging.info("handle finish message from WPS " + str(payload))
                    return busIndipendentMessages.FinishMessage(payload, msg['from'])
                elif ("topic=abort" in payload):
                    logging.info("handle abort message from WPS " + str(payload))
                    return busIndipendentMessages.AbortMessage(payload, msg['from'])
                elif ("topic=getloadavg" in payload):
                    return busIndipendentMessages.GetLoadAverageMessage(payload, msg['from'])
                else:
                    pass
        return None

    def SendMessage(self, message):
        m=self.Convert(message)
        m.send()

    def Stop(self, Message):
        pass

    def _register(self, iq):
        pass

    def Convert(self, busIndipendentMsg):
        if (type(busIndipendentMsg) is busIndipendentMessages.RegisterMessage):
            return xmppMessages.XMPPRegisterMessage(self, busIndipendentMsg.originator(), busIndipendentMsg.service, busIndipendentMsg.namespace, busIndipendentMsg.description, busIndipendentMsg.input_parameters(), busIndipendentMsg.output)

        if (type(busIndipendentMsg) is busIndipendentMessages.ProgressMessage):
            return xmppMessages.XMPPProgressMessage(  busIndipendentMsg.originator, self, busIndipendentMsg.progress )

        if (type(busIndipendentMsg) is busIndipendentMessages.LogMessage):
            return xmppMessages.XMPPLogMessage(busIndipendentMsg.originator, self, busIndipendentMsg.level, busIndipendentMsg.msg)

        if (type(busIndipendentMsg) is busIndipendentMessages.CompletedMessage):
            return xmppMessages.XMPPCompletedMessage(busIndipendentMsg.originator, self, busIndipendentMsg.base_url, busIndipendentMsg.outputs() )

        if (type(busIndipendentMsg) is busIndipendentMessages.ErrorMessage):
            return xmppMessages.XMPPErrorMessage(busIndipendentMsg.originator, self, busIndipendentMsg.msg, busIndipendentMsg.id)

        if (type(busIndipendentMsg) is busIndipendentMessages.AbortMessage):
            return xmppMessages.XMPPErrorMessage(busIndipendentMsg.originator, self, busIndipendentMsg.msg, busIndipendentMsg.id)

        if (type(busIndipendentMsg) is busIndipendentMessages.LoadAverageMessage):
            return xmppMessages.XMPPLoadAverageMessage(busIndipendentMsg.originator, self, busIndipendentMsg.outputs() )

        else:
            raise Exception("unknown message")


    def disconnect(self):
        self.xmpp.disconnect(wait=True)

    def state(self):
        return self.xmpp.state.current_state()
